from abetter.test.all_test_function import (
    mean_t_test,
    mean_z_test,
    prop_z_test,
    prop_fisher_test,
    var_f_test
)


__all__ = [
    'mean_t_test',
    'mean_z_test',
    'prop_z_test',
    'prop_fisher_test',
    'var_f_test',
]




"""
F检验：两独立样本方差是否相等F检验
"""

from scipy.stats import f as ff
from abetter.io.format import significance_type


# 两独立样本均值之差的z检验
def var_f_test_two_ind(n1: int, std1: float,
                       n2: int, std2: float,
                       h0: str = '==', alpha: float = 0.05) -> dict:
    """
    独立样本均值z检验
    H0: mean1 - mean2 - delta 与 0 的关系（==, <=, >=）
    :param n1: 左侧样本量
    :param std1: 左侧样本标准差
    :param n2: 右侧样本量
    :param std2: 右侧样本标准差
    :param h0: 原假设中的关系：μ1 - μ2 == δ, μ1 - μ2 <= δ, μ1 - μ2 >= δ
    :param alpha: α，第Ⅰ类错误概率（= 1 - 置信水平）
    :return: Z检验结果（dict）
    """
    # F 统计量
    f = std1 ** 2 / std2 ** 2
    # P值（z落在拒绝域的概率）
    if h0 in ('=', '=='):
        p_value = min(ff.sf(f, n1 - 1, n2 - 1), ff.cdf(f, n1 - 1, n2 - 1)) * 2
    elif h0 in ('<', '<='):
        p_value = ff.sf(f, n1 - 1, n2 - 1)
    elif h0 in ('>', '>='):
        p_value = ff.cdf(f, n1 - 1, n2 - 1)
    else:
        p_value = 'error'
    # 置信区间
    ci_f = list(ff.interval(1 - alpha / 2, n1 - 1, n2 - 1))

    return {
        'method': 'var_f_test',
        'method_name': '[独立样本][F检验]',
        'field_type': '均值型',
        'statistic': f,
        'p_value': p_value,
        'significance': significance_type(p_value),

        'diff': std1 - std2,  # 标准差的差异 = std1 - std2
        'ci_f': ci_f,         # 标准差比例的置信区间（双尾α）

        'n1': n1,
        'n2': n2,
        'std1': std1,
        'std2': std2,

        'test': f'({std1:.4f}) / ({std2:.4f}) {h0} 1',
        'h0': h0,
        'alpha': alpha
    }



import pandas as pd
import numpy as np


def data_wide_2_data_col(data: pd.DataFrame,
                         group_name: str = 'group_name',
                         sample_size: str = 'n',
                         mean_prefix: str = 'mean_',
                         std_prefix: str = 'std_',
                         k_prefix: str = 'k_'
                         ) -> pd.DataFrame:
    """
    宽数据转为列数据
    :param data: group_name, n, mean_xx, std_xx, mean_yy, std_yy, k_zz
    :param group_name:
    :param sample_size:
    :param mean_prefix:
    :param std_prefix:
    :param k_prefix:
    :return: group_name, field_type, field_name, n, mean, std, k
    """
    cols = data.columns
    if group_name != 'group_name':
        data.rename(columns={group_name: 'group_name'}, inplace=True)
    if sample_size != 'n':
        data.rename(columns={sample_size: 'n'}, inplace=True)
    if mean_prefix != 'mean_':
        n = len(mean_prefix)
        for col in cols:
            if col.startswith(mean_prefix):
                new_name = 'mean_' + col[n:]
                data.rename(columns={col: new_name}, inplace=True)
    if std_prefix != 'std_':
        n = len(std_prefix)
        for col in cols:
            if col.startswith(std_prefix):
                new_name = 'std_' + col[n:]
                data.rename(columns={col: new_name}, inplace=True)
    if k_prefix != 'k_':
        n = len(k_prefix)
        for col in cols:
            if col.startswith(k_prefix):
                new_name = 'k_' + col[n:]
                data.rename(columns={col: new_name}, inplace=True)

    field_name = [('mean', col[5:]) if col.startswith(mean_prefix) else ('prop', col[2:])
                  for col in cols if col.startswith(mean_prefix) or col.startswith(k_prefix)]

    row_num = len(data)
    temp = [pd.DataFrame({
        'group_name': data.group_name.values,
        'field_type': [f[0]] * row_num,
        'field_name': [f[1]] * row_num,
        'n': data.n.values,
        'mean': data['mean_' + f[1]].values if f[0] == 'mean' else [np.nan] * row_num,
        'std': data['std_' + f[1]].values if f[0] == 'mean' else [np.nan] * row_num,
        'k': data['k_' + f[1]].values if f[0] == 'prop' else [np.nan] * row_num,
    }) for f in field_name]

    return pd.concat(temp, axis=0)


def data_col_2_sample(data,
                      group_name: str = 'group_name',
                      field_type: str = 'field_type',
                      field_name: str = 'field_name',
                      sample_size: str='n',
                      mean: str='mean',
                      k: str='k', ):
    """
    列数据转化为 Sample
    :param data:
    :param group_name:
    :param field_type:
    :param field_name:
    :param sample_size:
    :param mean:
    :param k:
    :return:
    """
    data.rename(columns={
        group_name: 'group_name',
        field_type: 'field_type',
        field_name: 'field_name',
        sample_size: 'n',
        mean: 'mean',
        k: 'k'
    }, inplace=True)
    field_list = data.field_name.unique().tolist()
    group_list = data.group_name.unique().tolist()
    


"""
单指标样本及差值
"""
import pandas as pd

from .sample_basic import Sample, SS
from .test import prop_z_test, prop_fisher_test
from .io import ss_2_html, df_wide_2_df_stack


class SampleProp(Sample):
    """
    均值类型单指标样本
    """

    def __init__(self, n, k, alpha=0.05, beta=0.20, h0='==', days=None,
                 field_type='Prop', field_name=None, group_ratio=None, group_name=None):
        super().__init__(n=n, k=k, alpha=alpha, beta=beta, h0=h0, days=days,
                         field_type=field_type, field_name=field_name, group_ratio=group_ratio, group_name=group_name)
        self.n = n
        self.k = k
        self.mean = k / n

    def __sub__(self, other):
        if isinstance(other, SampleProp):
            return SSProp(self, other)
        elif isinstance(other, float):
            return '单样本检验，待补充'
        else:
            raise Exception("Invalid object!", other)


class SSProp(SS):
    """
    两个比率型指标样本的差
    """

    def __init__(self, s1: SampleProp, s2: SampleProp):
        super().__init__(s1, s2)
        self.dict_test_z = {}
        self.dict_test_fisher = {}
        self.dict_test_best = {}
        self.data_best = None
        self.data_flat = None
        self.data_all = None
        self.html_test_z = ''
        self.html_test_fisher = ''
        self.html_test_best = ''
        self.test()

    def test(self):
        self.dict_test_z = prop_z_test(s1=self.s1, s2=self.s2, alpha=self.s1.alpha, beta=self.s1.beta, h0=self.s1.h0, delta=0.0)
        self.dict_test_fisher = prop_fisher_test(s1=self.s1, s2=self.s2, alpha=self.s1.alpha, beta=self.s1.beta, h0=self.s1.h0, delta=0.0)

        if self.s1.k >= 10 and self.s1.n - self.s1.k >= 10 and self.s2.k >= 10 and self.s2.n - self.s2.k >= 10:
            self.dict_test_z.update({'最佳检验法': '是'})
            self.dict_test_fisher.update({'最佳检验法': '否'})
            self.dict_test_best = self.dict_test_z
            self.data_best = df_wide_2_df_stack(self.s1, self.s2, self.dict_test_z)
        else:
            self.dict_test_z.update({'最佳检验法': '不满足检验条件'})
            self.dict_test_fisher.update({'最佳检验法': '是'})
            self.dict_test_best = self.dict_test_fisher
            self.data_best = df_wide_2_df_stack(self.s1, self.s2, self.dict_test_fisher)
        self.data_flat = pd.DataFrame([self.dict_test_z, self.dict_test_fisher])
        self.data_all = pd.concat([
            df_wide_2_df_stack(s1=self.s1, s2=self.s2, test_data=self.dict_test_z),
            df_wide_2_df_stack(s1=self.s1, s2=self.s2, test_data=self.dict_test_fisher)
        ], axis=0)
        self.html_test_z = ss_2_html(s1=self.s1, s2=self.s2, test_data=self.dict_test_z)
        self.html_test_fisher = ss_2_html(s1=self.s1, s2=self.s2, test_data=self.dict_test_fisher)
        self.html_test_best = ss_2_html(s1=self.s1, s2=self.s2, test_data=self.dict_test_best)

    def _repr_html_(self):
        return self.html_test_best


__all__ = ['SampleProp', 'SSProp']



"""
t检验：两配对样本均值之差t检验
"""

from scipy.stats import t as tf
import numpy as np
from abetter.io.format import significance_type


# 配对样本均值的t检验
def mean_t_test_two_pair(n: int, diff: float, diff_std: float,
                         diff0: float, h0: str = '==', alpha=0.05, beta=0.20) -> dict:
    # 自由度
    df = n - 1
    # 上分位点
    t_upp_2tail_alpha = tf.isf(alpha / 2, df=df)
    t_upp_1tail_alpha = tf.isf(alpha, df=df)
    t_upp_1tail_beta = tf.isf(beta, df=df)
    # 统计量
    sem = diff_std / np.sqrt(n)
    diff_mean = diff - diff0
    t = diff_mean / sem
    # P值（z落在拒绝域的概率）
    if h0 in ('=', '=='):
        p_value = tf.sf(abs(t), df=df) * 2
    elif h0 in ('<', '<='):
        p_value = tf.sf(t, df=df)
    elif h0 in ('>', '>='):
        p_value = tf.cdf(t, df=df)
    else:
        p_value = '错误'
    # 置信区间半径
    ci_diff_mean_radius = t_upp_2tail_alpha * sem
    ci_diff_mean = list(diff + np.array([-1, 1]) * ci_diff_mean_radius)
    # MDE
    mde_mean = (t_upp_2tail_alpha + t_upp_1tail_beta) * sem
    # effect_size
    effect_size = abs(diff) / diff_std
    return {
        'method': 'mean_pair_t_test',
        'method_name': '[配对样本][t检验]',
        'field_type': '均值型',
        't': t,
        'p_value': p_value,
        'significance': significance_type(p_value),
        'diff': diff,                            # 均值的增量
        'incr_mean': diff,                       # 均值的增量 = mean1 - mean2
        'incr_group1': diff * n,                 # 组1的增量 = (mean1 - mean2) * n，这里n1=n2
        'incr_group2': diff * n,                 # 组1的增量 = (mean1 - mean2) * n，这里n1=n2

        'mde': mde_mean,                         # 均值增量MDE
        'ci': ci_diff_mean,                      # 均值增量的置信区间（双尾α）
        'ci_radius': ci_diff_mean_radius,        # 均值增量的置信区间半径（双尾α）
        'effect_size': effect_size,              # 效应量（Cohen's d）

        'std': diff_std,                         # 标准差（复制一份）
        'sem': sem,                              # 均值的标准误

        'n': n,
        'diff0': diff0,
        'diff_std': diff_std,

        'test': f'{diff:.4f} {h0} {diff0:.4f}',
        'h0': h0,
        'mean0': diff0,
        'alpha': alpha,
        'beta': beta
    }



"""
单指标样本及差值
"""
import pandas as pd

from .sample_basic import Sample, SS
from .test import mean_t_test, mean_z_test, var_f_test
from .io import ss_2_html, df_wide_2_df_stack


class SampleMean(Sample):
    """
    均值类型单指标样本
    """

    def __init__(self, n, mean, std, alpha=0.05, beta=0.20, h0='==', days=None,
                 field_type='Mean', field_name=None, group_ratio=None, group_name=None):
        super().__init__(n=n, mean=mean, std=std, alpha=alpha, beta=beta, h0=h0, days=days,
                         field_type=field_type, field_name=field_name, group_ratio=group_ratio, group_name=group_name)

    def __sub__(self, other):
        if isinstance(other, SampleMean):
            return SSMean(self, other)
        elif isinstance(other, float) or isinstance(other, int):
            return '单样本检验，待补充'
        else:
            raise Exception("Invalid object!", other)


class SSMean(SS):
    """
    两个均值型单指标样本的差
    """

    def __init__(self, s1: SampleMean, s2: SampleMean):
        super().__init__(s1, s2)

        self.dict_test_best = {}
        self.dict_test_z = {}
        self.dict_test_t_eq_var = {}
        self.dict_test_t_ne_var = {}
        self.dict_equal_var = {}

        self.data_best = None
        self.data_flat = None
        self.data_all = None

        self.html_test_best = ''
        self.html_test_z = ''
        self.html_test_t_eq_var = ''
        self.html_test_t_ne_var = ''

        self.test()

    def test(self):
        self.dict_test_z = mean_z_test(s1=self.s1, s2=self.s2, alpha=self.s1.alpha, beta=self.s1.beta, h0=self.s1.h0, delta=0.0)
        self.dict_test_t_eq_var = mean_t_test(s1=self.s1, s2=self.s2, alpha=self.s1.alpha, beta=self.s1.beta, h0=self.s1.h0, delta=0.0, equal_var=True)
        self.dict_test_t_ne_var = mean_t_test(s1=self.s1, s2=self.s2, alpha=self.s1.alpha, beta=self.s1.beta, h0=self.s1.h0, delta=0.0, equal_var=False)

        self.dict_test_z.update({'最佳检验法': ''})
        self.dict_test_t_eq_var.update({'最佳检验法': ''})
        self.dict_test_t_ne_var.update({'最佳检验法': ''})
        self.dict_equal_var = var_f_test(s1=self.s1, s2=self.s2, h0='==')
        if self.s1.n > 30 and self.s2.n > 30:
            self.dict_test_z.update({'最佳检验法': '是'})
            self.dict_test_best = self.dict_test_z
        elif self.dict_equal_var.get('p_value') < self.s1.alpha:
            # 方差不相等
            self.dict_test_t_ne_var.update({'最佳检验法': '是'})
            self.dict_test_best = self.dict_test_t_ne_var
        else:
            # 方差相等
            self.dict_test_t_eq_var.update({'最佳检验法': '是'})
            self.dict_test_best = self.dict_test_t_eq_var
        self.data_best = df_wide_2_df_stack(self.s1, self.s2, self.dict_test_best)
        self.data_flat = pd.DataFrame([self.dict_test_z, self.dict_test_t_eq_var, self.dict_test_t_ne_var])
        self.data_all = pd.concat([
            df_wide_2_df_stack(s1=self.s1, s2=self.s2, test_data=self.dict_test_z),
            df_wide_2_df_stack(s1=self.s1, s2=self.s2, test_data=self.dict_test_t_eq_var),
            df_wide_2_df_stack(s1=self.s1, s2=self.s2, test_data=self.dict_test_t_ne_var)
        ], axis=0)

        self.html_test_best = ss_2_html(s1=self.s1, s2=self.s2, test_data=self.dict_test_best)
        self.html_test_z = ss_2_html(s1=self.s1, s2=self.s2, test_data=self.dict_test_z)
        self.html_test_t_eq_var = ss_2_html(s1=self.s1, s2=self.s2, test_data=self.dict_test_t_eq_var)
        self.html_test_t_ne_var = ss_2_html(s1=self.s1, s2=self.s2, test_data=self.dict_test_t_ne_var)

    def _repr_html_(self):
        return self.html_test_best


__all__ = ['SampleMean', 'SSMean']



import pandas as pd
from abetter.sample_basic import Sample
from abetter.io import to_html
from abetter.sample_basic import Sample


def to_str(v, n=4) -> str:
    if v is None:
        return ''
    if isinstance(v, str):
        return v
    if abs(v) > 1000:
        return f'{v:.0f}'
    if abs(v) > 100:
        return f'{v:.1f}'
    if abs(v) > 10:
        return f'{v:.2f}'
    if abs(v) > 0:
        return f'{v:.3f}'
    if abs(v) > 0.1:
        return f'{v:.4f}'
    if abs(v) > 0.01:
        return f'{v:.5f}'
    return f'{v:.6f}'


def significance_color(pv: float) -> str:
    if pv < 0.01:
        return 'bg2'
    if pv < 0.05:
        return 'bg1'
    return 'bg0'


def significance_type(pv: float) -> str:
    if pv < 0.01:
        return '显著**'
    elif pv < 0.05:
        return '显著*'
    else:
        return '不显著'


def none_str(a, b, c=-1):
    """
    返回 a 或 a*b 或 a*b/c
    :param a:
    :param b:
    :param c:
    :return: a 或 a*b 或 a*b/c
    """
    if a == '':
        return ''
    elif isinstance(a, str):
        return a
    elif c is None:
        return '[未定义分组比例]'
    elif c > 0:
        return a * b / c
    elif c == -1:
        return a * b
    else:
        return '[未定义分组比例]'

def get_head() -> list[dict]:
    row1 = [
        {'element': 'th', 'col': 2, 'cls': 'bt bl br', 'style': 'text-align: center', 'data': '指标'},
        {'element': 'th', 'col': 2, 'cls': 'bt bl br', 'style': 'text-align: center', 'data': '样本'},
        {'element': 'th', 'col': 2, 'cls': 'bt bl br', 'style': 'text-align: center', 'data': '实验参数'},
        {'element': 'th', 'col': 2, 'cls': 'bt bl br', 'style': 'text-align: center', 'data': '统计量'},
        {'element': 'th', 'col': 5, 'cls': 'bt bl br', 'style': 'text-align: center', 'data': '增量效果'}
    ]
    row2 = [
        {'cls': 'bl bb', 'style': 'text-align: center', 'data': '指标名称'}, {'cls': 'br bb', 'style': 'text-align: center', 'data': '指标类型'},
        {'cls': 'bl bb', 'style': 'text-align: center', 'data': '名称'}, {'cls': 'br bb', 'style': 'text-align: center', 'data': '值'},
        {'cls': 'bl bb', 'style': 'text-align: center', 'data': '名称'}, {'cls': 'br bb', 'style': 'text-align: center', 'data': '值'},
        {'cls': 'bl bb', 'style': 'text-align: center', 'data': '名称'}, {'cls': 'br bb', 'style': 'text-align: center', 'data': '值'},
        {'cls': 'bl bb', 'style': 'text-align: center', 'data': '名称'},
        {'cls': 'bb', 'style': 'text-align: center', 'data': '样本均值增量'},
        {'cls': 'bb', 'style': 'text-align: center', 'data': '左组增量'},
        {'cls': 'bb', 'style': 'text-align: center', 'data': '右组增量'},
        {'cls': 'br bb', 'style': 'text-align: center', 'data': '推全增量'}
    ]
    return [{'data': row1}, {'data': row2}]


def get_body(s1, s2, test_data: dict) -> list[dict]:
    field_name = s1.field_name
    field_type = test_data.get('field_type')
    n1, n2, m1, m2 = s1.n, s2.n, s1.mean, s2.mean
    gr1, gr2 = s1.group_ratio, s2.group_ratio
    b_color = significance_color(test_data.get('p_value'))
    row1 = [
        {'row': 4, 'cls': f'bl br bb {b_color}', 'style': 'text-align: center', 'data': f'{field_name or ''}'},
        {'row': 4, 'cls': 'bl br bb', 'style': 'text-align: center', 'data': f'{field_type}'},
        {'cls': 'bl', 'style': 'text-align: right; color: #666', 'data': '左组样本量'}, {'cls': 'br', 'style': 'text-align: right;', 'data': f'{n1:,.0f}'},
        {'cls': 'bl', 'style': 'text-align: right; color: #666', 'data': '检验法'}, {'cls': 'br', 'style': 'text-align: right;', 'data': f'{test_data.get('method_name')}'},
        {'cls': 'bl', 'style': 'text-align: right; color: #666', 'data': '显著性'}, {'cls': f'br {b_color}', 'style': 'text-align: right;', 'data': f'{test_data.get('significance')}'},
        {'cls': 'bl', 'style': 'text-align: right; color: #666', 'data': '增量'},
        {'style': 'text-align: right;', 'data': f'{to_str(test_data.get('incr_mean'))}'},
        {'style': 'text-align: right;', 'data': f'{to_str(test_data.get('incr_group1'))}'},
        {'style': 'text-align: right;', 'data': f'{to_str(test_data.get('incr_group2'))}'},
        {'cls': 'br', 'style': 'text-align: right;', 'data': f'{to_str(none_str(test_data.get('incr_group1'), 1,  gr1))}'}
    ]
    row2 = [
        {'cls': 'bl', 'style': 'text-align: right; color: #666', 'data': '右组样本量'}, {'cls': 'br', 'style': 'text-align: right;', 'data': f'{n2:,.0f}'},
        {'cls': 'bl', 'style': 'text-align: right; color: #666', 'data': 'α'}, {'cls': 'br', 'style': 'text-align: right;', 'data': f'{to_str(test_data.get('alpha'))}'},
        {'cls': 'bl', 'style': 'text-align: right; color: #666', 'data': 'P Value'}, {'cls': 'br', 'style': 'text-align: right;', 'data': f'{to_str(test_data.get('p_value'))}'},
        {'cls': 'bl', 'style': 'text-align: right; color: #666', 'data': 'MDE'},
        {'style': 'text-align: right;', 'data': f'{to_str(test_data.get('mde'))}'},
        {'style': 'text-align: right;', 'data': f'{to_str(none_str(test_data.get('mde'), n1))}'},
        {'style': 'text-align: right;', 'data': f'{to_str(none_str(test_data.get('mde'), n2))}'},
        {'cls': 'br', 'style': 'text-align: right;', 'data': f'{to_str(none_str(test_data.get('mde') , n1 , gr1))}'}
    ]
    row3 = [
        {'cls': 'bl', 'style': 'text-align: right; color: #666', 'data': '左组均值'}, {'cls': 'br', 'style': 'text-align: right;', 'data': f'{to_str(m1)}'},
        {'cls': 'bl', 'style': 'text-align: right; color: #666', 'data': 'β'}, {'cls': 'br', 'style': 'text-align: right;', 'data': f'{to_str(test_data.get('beta'))}'},
        {'cls': 'bl', 'style': 'text-align: right; color: #666', 'data': 'Statistic'}, {'cls': 'br', 'style': 'text-align: right;', 'data': f'{to_str(test_data.get('statistic'))}'},
        {'cls': 'bl', 'style': 'text-align: right; color: #666', 'data': 'CI Lower'},
        {'style': 'text-align: right;', 'data': f'{to_str(test_data.get('ci')[0])}'},
        {'style': 'text-align: right;', 'data': f'{to_str(none_str(test_data.get('ci')[0], n1))}'},
        {'style': 'text-align: right;', 'data': f'{to_str(none_str(test_data.get('ci')[0], n2))}'},
        {'cls': 'br', 'style': 'text-align: right;', 'data': f'{to_str(none_str(test_data.get('ci')[0], n1, gr1))}'}
    ]
    row4 = [
        {'cls': 'bl bb', 'style': 'text-align: right; color: #666', 'data': '右组均值'}, {'cls': 'br bb', 'style': 'text-align: right;', 'data': f'{to_str(m2)}'},
        {'cls': 'bl bb', 'style': 'text-align: right; color: #666', 'data': '$H_0$'}, {'cls': 'br bb', 'style': 'text-align: right;', 'data': f'{test_data.get('h0')}'},
        {'cls': 'bl bb', 'style': 'text-align: right; color: #666', 'data': 'Effect Size'}, {'cls': 'br bb', 'style': 'text-align: right;', 'data': f'{to_str(test_data.get('effect_size'))}'},
        {'cls': 'bl bb', 'style': 'text-align: right; color: #666', 'data': 'CI Upper'},
        {'cls': 'bb', 'style': 'text-align: right;', 'data': f'{to_str(test_data.get('ci')[1])}'},
        {'cls': 'bb', 'style': 'text-align: right;', 'data': f'{to_str(none_str(test_data.get('ci')[1], n1))}'},
        {'cls': 'bb', 'style': 'text-align: right;', 'data': f'{to_str(none_str(test_data.get('ci')[1], n2))}'},
        {'cls': 'br bb', 'style': 'text-align: right;', 'data': f'{to_str(none_str(test_data.get('ci')[1], n1, gr1))}'}
    ]
    return [{'data': row1}, {'data': row2}, {'data': row3}, {'data': row4}]


def ss_2_html(s1: Sample, s2: Sample, test_data: dict) -> str:
    _caption = {'data': [
        {'data': 'AB实验结果', 'style': 'text-align: center; margin: 1px; font-weight: 600;'},
        {'data': f'{s1.group_name if s1.group_name else '左组'} - {s2.group_name if s2.group_name else '右组'}', 'style': 'text-align: center; margin: 1px'}]}
    _head = {'data': get_head()}
    _body = {'data': get_body(s1, s2, test_data)}
    _foot = None

    return to_html.html_table(caption=_caption, head=_head, body=_body, foot=_foot)


def df_wide_2_df_stack(s1: Sample, s2: Sample, test_data: dict) -> pd.DataFrame:
    """
    df宽表 → df可读表
    :param s1: 样本1
    :param s2: 样本2
    :param test_data: 检验数据
    :return: 可读 DataFrame
    """
    _method_name = test_data.get('method_name')
    _is_beast = test_data.get('最佳检验法')
    _index = [(_method_name, _is_beast, 1), (_method_name, _is_beast, 2), (_method_name, _is_beast, 3), (_method_name, _is_beast, 4)]
    _columns = [('指标', '指标名称'), ('指标', '指标类型'),
                ('样本', '名称'), ('样本', '值'),
                ('实验参数', '名称'), ('实验参数', '值'),
                ('统计量', '名称'), ('统计量', '值'),
                ('增量效果', '名称'), ('增量效果', '样本均值增量'), ('增量效果', '左组增量'), ('增量效果', '右组增量'), ('增量效果', '推全增量')]
    _d = test_data
    _field_name, _field_type = s1.field_name if s1.field_name is not None else '[未定义指标名称]', '[均值型]' if s1.field_type=='Mean' else '[比率型]'
    _gr1, _gr2 = s1.group_ratio, s2.group_ratio
    _mde = _d.get('mde', '')
    _cil, _ciu = _d.get('ci', ('', ''))

    return pd.DataFrame([
        # 第1行
        [_field_name, _field_type, '左组样本量', s1.n, '检验法', _d.get('method_name'), '显著性', _d.get('significance'),
         '增量', _d.get('incr_mean'), _d.get('incr_group1'), _d.get('incr_group2'), none_str(_d.get('incr_group1'), 1, _gr1)],
        # 第2行
        [_field_name, _field_type, '右组样本量', s2.n, 'α', _d.get('alpha'), 'P Value', _d.get('p_value'),
         'MDE', _mde, none_str(_mde, s1.n), none_str(_mde, s2.n), none_str(_mde, s1.n, _gr1)],
        # 第3行
        [_field_name, _field_type, '左组均值', s1.mean if _field_type == '[均值型]' else s1.k / s1.n, 'β', _d.get('beta'), 'Statistic', _d.get('statistic'),
         'CI Lower', _cil, none_str(_cil, s1.n), none_str(_cil, s2.n), none_str(_cil, s1.n, _gr1)],
        # 第4行
        [_field_name, _field_type, '左组均值', s1.mean if _field_type == '[均值型]' else s1.k / s1.n, '$H_0$', _d.get('h0'), 'Effect Size', _d.get('effect_size'),
         'CI Upper', _ciu, none_str(_ciu, s1.n), none_str(_ciu, s2.n), none_str(_ciu, s1.n, _gr1)]
    ], index=pd.MultiIndex.from_tuples(_index, names=['检验法','是否最佳','序号']), columns=pd.MultiIndex.from_tuples(_columns))



"""
输出HTML表格
"""

def html_td(data: str, element: str = 'td', row: int = None, col: int = None, style: str = None, cls: str = None):
    """
    生成1个单元格
    :param data: 已经格式化好的字符串（可以包含HTML标签）
    :param element: td 或 th，如果是 None 使用 td
    :param row: 跨行行数
    :param col: 跨列列数
    :param style: 指定单元格 样式
    :param cls: 指定单元格 class
    :return:
    """
    _html = f"""<{element or 'td'}
{' rowspan="' + str(row) + '"' if row else ''}
{' colspan="' + str(col) + '"' if col else ''}
{' class="' + cls + '"' if cls else ''}
{' style="' + style + '"' if style else ''}
>{data}</{element or 'td'}>
"""
    return _html.replace('\n', '')


def html_tr(data: list[dict], style=None, cls=None):
    """
    生成1行tr
    :param data: 列表，每个元素是一个td数据
     例如：[{'data': 'xxx', 'element': 'td', 'row': 2, 'col': None, 'cls': 'c1.c2.c3', 'style': 'color: #F00; background: #FFF;'},...]
    :param style: 指定 tr 样式
    :param cls: 指定 tr 类
    :return:
    """
    if data is None:
        return ''
    _html = ''
    for da in data:
        _td = html_td(element=da.get('element'), data=da.get('data'), row=da.get('row'), col=da.get('col'), cls=da.get('cls'), style=da.get('style'))
        _html += _td + '\n'
    return f"""<tr{' class="' + cls + '"' if cls else ''}{' style="' + style + '"' if style else ''}>{_html}</tr>"""


def html_caption(data: list[dict], cls=None, style=None):
    """
    生成表标题
    :param data:
    :param cls:
    :param style:
    :return:
    """
    _caption = ''
    _row = ''
    for da in data:
        _row += f"""<p{' class="' + da.get('cls') + '"' if da.get('cls') else ''}{' style="' + da.get('style') + '"'}>{da.get('data')}</p>"""
    if _row != '':
        _caption += f"""<caption{' class="' + cls + '"' if cls else ''}{' style="' + style + '"' if style else ''}>{_row}</caption>"""
    return _caption


def html_body(element: str = 'tbody', data: list[dict] = None, cls=None, style=None):
    """
    生成多行tr，即：thaed, tbody, tfoot内部HTML
    :param element: thaed, tbody, tfoot
    :param data: 列表，每个元素是一个tr数据，例如：[{'data': 'xxx', 'cls': 'c1.c2.c3', 'style': 'color: #F00; background: #FFF;'},...]
    :param cls:
    :param style:
    :return:
    """
    _body = ''
    _row = ''
    for da in data:
        _row += html_tr(data=da.get('data'), cls=da.get('cls'), style=da.get('style')) + '\n'
    if _row != '':
        _body += f"""<{element}{' class="' + cls + '"' if cls else ''}{' style="' + style + '"' if style else ''}>{_row}</{element}>"""
    return _body


def html_table(caption: dict = None,
               head: dict = None,
               body: dict = None,
               foot: dict = None,
               css: str = None, cls: str = None, style: str = 'border="1" class="dataframe"'):
    """
    生成Table
    :param caption: 字典，例如：{'data': 'xxx', 'cls': 'c1.c2.c3', 'style': 'color: #F00; background: #FFF;'}
    :param head: 字典，例如：{'data': 'xxx', 'cls': 'c1.c2.c3', 'style': 'color: #F00; background: #FFF;'}。
    其中data是一个列表，每个元素是一个tr数据，例如：[{'data': 'xxx', 'cls': 'c1.c2.c3', 'style': 'color: #F00; background: #FFF;'},...]
    :param body: 字典，同上
    :param foot: 字典，同上
    :param css: 指定 CSS
    :param cls: 指定 table 类
    :param style: 指定 table 样式
    :return:
    """
    # 标题
    _css = css or """<style scoped>
th.ba, td.ba {border: 1px solid #ccc}
th.blr, td.blr {border-left: 1px solid #ccc; border-right: 1px solid #ccc;}
th.btb, td.btb {border-top: 1px solid #ccc; border-bottom: 1px solid #ccc;}
th.bl, td.bl {border-left: 1px solid #ccc}
th.br, td.br {border-right: 1px solid #ccc}
th.bt, td.bt {border-top: 1px solid #ccc}
th.bb, td.bb {border-bottom: 1px solid #ccc}

th.bg0, td.bg0 {background-color: #cccccc}
th.bg1, td.bg1 {background-color: #fcf6bd}
th.bg2, td.bg2 {background-color: #ffcb05}
th.c, td.c {text-align: center}
</style>"""
    # 表 caption
    _caption = ''
    if caption is not None:
        _caption = html_caption(data=caption.get('data'), cls=caption.get('cls'), style=caption.get('style'))
    # 表 head
    _head = ''
    if head is not None:
        _head = html_body(element='thead', data=head.get('data'), cls=head.get('cls'), style=head.get('style'))

    # 表 body
    _body = ''
    if body is not None:
        _body = html_body(element='tbody', data=body.get('data'), cls=body.get('cls'), style=body.get('style'))

    # 表 foot
    _foot = ''
    if foot is not None:
        _foot = html_body(element='tfoot', data=foot.get('data'), cls=foot.get('cls'), style=foot.get('style'))
    # 整合

    # 返回结果
    return f"""<div>
{_css}
<table {'class="' + cls + '"' if cls else ''} {'style="' + style + '"' if style else ''}>
{_caption}
{_head}
{_body}
{_foot}
</table></div>
"""



from matplotlib import pyplot as plt
from scipy.stats import norm
import numpy as np
from abetter import SampleMean
from abetter.sample_mean import SSMean
from abetter.io.format import to_str


def plot_two_mean(s1: SampleMean, s2: SampleMean, points: int = 200) -> tuple[plt.Figure, plt.Axes]:
    """
    绘制两个均值型样本的总体均值分布
    :param s1: 样本1
    :param s2: 样本2
    :param points: 绘图精度，越大精度越高，默认200
    :return:
    """
    with plt.ioff():
        c1, c2 = '#118ab2', '#bc4749'
        # 计算
        std1, std2 = s1.std / np.sqrt(s1.n), s2.std / np.sqrt(s2.n)
        x_min = min(s1.mean - std1 * 5, s2.mean - std2 * 5)
        x_max = max(s1.mean + std1 * 5, s2.mean + std2 * 5)
        x = np.linspace(x_min, x_max, points)
        y1 = norm.pdf(x, loc=s1.mean, scale=std1)
        y2 = norm.pdf(x, loc=s2.mean, scale=std2)
        y_max = np.max([y1, y2]) * 1.1

        # 绘制密度函数
        fig, ax = plt.subplots(figsize=(7, 5))
        ax.plot(x, 0 * x, c='k', lw=1)

        sd1, = ax.plot(x, y1, c=c1, label=r'$\overline{X}_1$')
        sd2, = ax.plot(x, y2, c=c2, label=r'$\overline{X}_2$')
        ax.set_xlim(x_min, x_max)

        # 绘制辅助线
        ax.vlines(s1.mean, 0, y_max, lw=0.5, ls='-', color='k', alpha=0.5)
        ax.vlines(s2.mean, 0, y_max, lw=0.5, ls='-', color='k', alpha=0.5)
        ax.hlines(y_max * 0.95, s1.mean, s2.mean, lw=1, ls='-', color='k', alpha=0.5)
        ax.text((s1.mean + s2.mean) / 2, y_max * 0.95, f'Diff={to_str(s2.mean - s1.mean)}', ha='center', va='bottom')

        # 填充
        alpha = 0.05 if s1.alpha is None else s1.alpha
        label1, label2 = f'{100 * (1 - alpha):.0f}%' + r' CI of $\overline{X}_1$', f'{100 * (1 - alpha):.0f}%' + r' CI of $\overline{X}_2$'
        z = norm.isf(alpha / 2)
        x1 = np.linspace(s1.mean - std1 * z, s1.mean + std1 * z, points)
        y1 = norm.pdf(x1, loc=s1.mean, scale=std1)
        x2 = np.linspace(s2.mean - std2 * z, s2.mean + std2 * z, points)
        y2 = norm.pdf(x2, loc=s2.mean, scale=std2)
        f1 = ax.fill_between(x1, y1, 0, color=c1, alpha=0.5, label=label1)
        f2 = ax.fill_between(x2, y2, 0, color=c2, alpha=0.5, label=label2)
        ax.text(s1.mean, np.max(y1) * 0.3, '95%', ha='center', va='center')
        ax.text(s2.mean, np.max(y2) * 0.3, '95%', ha='center', va='center')

        # 坐标刻度
        ax.set_frame_on(False)
        ax.set_axis_off()

        ax.vlines(s1.mean, 0 - y_max * 0.02, 0, lw=0.5, ls='-', color='k', alpha=0.5)
        ax.text(s1.mean, 0 - y_max * 0.02, to_str(s1.mean), ha='center', va='top')

        ax.vlines(s2.mean, 0 - y_max * 0.06, 0, lw=0.5, ls='-', color='k', alpha=0.5)
        ax.text(s2.mean, 0 - y_max * 0.06, to_str(s2.mean), ha='center', va='top')

        # 设置
        plt.title('Distributions of 2 Sample Means')
        ax.legend([sd1, sd2, f1, f2], [r'$\overline{X}_1$', r'$\overline{X}_2$', label1, label2])

        return fig, ax


def plot_diff_mean(ss: SSMean, points: int = 200) -> tuple[plt.Figure, plt.Axes]:
    with plt.ioff():
        c1, c2 = '#118ab2', '#bc4749'
        # 计算
        sem = ss.dict_test_best.get('sem')
        mde = ss.dict_test_best.get('mde')
        alpha = 0.05 if ss.s1.alpha is None else ss.s1.alpha
        beta = 0.20 if ss.s1.beta is None else ss.s1.beta
        z = norm.isf(alpha / 2)

        x_min, x_max = -sem * 5, mde + sem * 5
        x = np.linspace(x_min, x_max, points)
        y1 = norm.pdf(x, loc=0.0, scale=sem)
        y2 = norm.pdf(x, loc=mde, scale=sem)
        y_max = np.max([y1, y2])

        # 绘制密度函数
        fig, ax = plt.subplots(figsize=(8, 5))
        ax.plot(x, 0 * x, c='k', lw=1)
        sd1, = ax.plot(x, y1, c=c1, label='H0')
        sd2, = ax.plot(x, y2, c=c2, label='H1', ls='--')

        # 绘制辅助线
        ax.vlines(0.0, 0, y_max, lw=0.5, ls='-', color='k', alpha=0.5)
        ax.vlines(mde, 0, y_max, lw=0.5, ls='-', color='k', alpha=0.5)
        ax.vlines(sem * z, 0, y_max * 1.05, lw=0.5, ls='-', color='k', alpha=0.5)
        ax.text(sem * z, y_max * 1.05, f'c={to_str(sem * z)}', ha='center', va='bottom')

        # 填充
        label1, label2 = r'Error Ⅰ: $\alpha={alpha}$'.format(alpha=f'{alpha: .2f}'), r'Error Ⅱ: $\beta={beta}$'.format(beta=f'{beta: .2f}')
        x1 = np.linspace(sem * z, sem * z * 5, points)
        y1 = norm.pdf(x1, loc=0, scale=sem)
        x2 = np.linspace(-sem * z * 5, sem * z, points)
        y2 = norm.pdf(x2, loc=mde, scale=sem)
        f1 = ax.fill_between(x1, y1, 0, color=c1, alpha=0.5, label=label1)
        f2 = ax.fill_between(x2, y2, 0, color=c2, alpha=0.5, label=label2)

        # 坐标刻度
        ax.set_frame_on(False)
        ax.set_axis_off()
        ax.vlines(0, 0 - y_max * 0.02, 0, lw=0.5, ls='-', color='k', alpha=0.5)
        ax.text(0, 0 - y_max * 0.02, '0', ha='center', va='top')

        ax.vlines(mde, 0 - y_max * 0.06, 0, lw=0.5, ls='-', color='k', alpha=0.5)
        ax.text(mde, 0 - y_max * 0.06, f'MDE={to_str(mde)}', ha='center', va='top')
        ax.text(sem * z + mde * 0.05, y_max * 0.03, r'$\alpha=${alpha}%'.format(alpha=f'{100 * alpha:.0f}'), ha='left', va='center')
        ax.text(sem * z - mde * 0.05, y_max * 0.03, r'$\beta=${beta}%'.format(beta=f'{100 * beta:.0f}'), ha='right', va='center')

        # 设置
        plt.title('Distribution of the Difference Between 2 Sample Means')
        ax.set_xlim(x_min, x_max)
        ax.set_ylim(-y_max * 0.06, y_max * 1.15)
        ax.legend([sd1, sd2, f1, f2], [
            r'$H_0: \overline{X}_2$ - $\overline{X}_1 = 0$',
            r'$H_1: \overline{X}_2$ - $\overline{X}_1 \geqslant MDE$',
            r'Error Ⅰ: $\alpha=${alpha}'.format(alpha=f'{100*alpha:.0f}%'),
            r'Error Ⅱ: $\beta=${beta}'.format(beta=f'{100*beta:.0f}%')
        ])
        return fig, ax


"""
z检验：两独立样本均值之差Z检验
"""

from scipy.stats import norm
import numpy as np
from abetter.io.format import significance_type


# 两独立样本均值之差的z检验
def mean_z_test_two_ind(n1: int, mean1: float, std1: float,
                        n2: int, mean2: float, std2: float,
                        delta: float = 0.0,
                        h0: str = '==', alpha: float = 0.05, beta: float = 0.20) -> dict:
    """
    独立样本均值z检验
    H0: mean1 - mean2 - delta 与 0 的关系（==, <=, >=）
    :param n1: 左侧样本量
    :param mean1: 左侧样本均值
    :param std1: 左侧样本标准差
    :param n2: 右侧样本量
    :param mean2: 右侧样本均值
    :param std2: 右侧样本标准差
    :param delta: 原假设中两均值的差值δ
    :param h0: 原假设中的关系：μ1 - μ2 == δ, μ1 - μ2 <= δ, μ1 - μ2 >= δ
    :param alpha: α，第Ⅰ类错误概率（= 1 - 置信水平）
    :param beta: β，第Ⅱ类错误概率（= 1 - 功效）
    :return: Z检验结果（dict）
    """
    # 上分位点
    z_upp_2tail_alpha = norm.isf(alpha / 2)
    z_upp_1tail_alpha = norm.isf(alpha)
    z_upp_1tail_beta = norm.isf(beta)
    # SEM 样本均值的标准误
    sem = np.sqrt(std1 ** 2 / n1 + std2 ** 2 / n2)
    # 合并标准差
    std = np.sqrt(((n1 - 1) * std1 ** 2 + (n2 - 1) * std2 ** 2) / (n1 - 1 + n2 - 1))
    # 均值差
    diff_mean = mean1 - mean2
    # z 统计量
    z = (diff_mean - delta) / sem
    # P值（z落在拒绝域的概率）
    if h0 in ('=', '=='):
        p_value = norm.sf(abs(z)) * 2
    elif h0 in ('<', '<='):
        p_value = norm.sf(z)
    elif h0 in ('>', '>='):
        p_value = norm.cdf(z)
    else:
        p_value = 'error'
    # 置信区间半径
    ci_diff_mean_radius = z_upp_2tail_alpha * sem
    ci_diff_mean = list(diff_mean + np.array([-1, 1]) * ci_diff_mean_radius)
    # MDE
    mde_mean = (z_upp_2tail_alpha + z_upp_1tail_beta) * sem
    # effect_size
    effect_size = abs(diff_mean) / std
    return {
        'method': 'mean_z_test',
        'method_name': '[独立样本][z检验]',
        'field_type': '均值型',
        'statistic': z,
        'p_value': p_value,
        'significance': significance_type(p_value),

        'diff': diff_mean,  # 均值的增量 = mean1 - mean2
        'incr_mean': diff_mean,  # 均值的增量 = mean1 - mean2
        'incr_group1': diff_mean * n1,           # 组1的增量 = (mean1 - mean2) * n1
        'incr_group2': diff_mean * n2,           # 组2的增量 = (mean1 - mean2) * n2，如果组2也做干预的话
        'mde': mde_mean,  # 均值增量MDE
        'ci': ci_diff_mean,  # 均值增量的置信区间（双尾α）
        'ci_radius': ci_diff_mean_radius,  # 均值增量的置信区间半径（双尾α）
        'effect_size': effect_size,  # 效应量（Cohen's d）

        'std': std,  # 池化标准差（两标准差的加权平均值）
        'sem': sem,  # 均值的标准误

        'n1': n1,
        'n2': n2,
        'mean1': mean1,
        'mean2': mean2,
        'std1': std1,
        'std2': std2,

        'test': f'({mean1:.4f}) - ({mean2:.4f}) {h0} {delta:.4f}',
        'h0': h0,
        'alpha': alpha,
        'beta': beta
    }



"""
z检验：两独立样本比率之差Z检验
"""

from scipy.stats import fisher_exact
import numpy as np
from abetter.io.format import significance_type


# 两独立样本均值之差的z检验
def prop_fisher_exact_test_two_ind(n1: int, k1: float,
                                   n2: int, k2: float,
                                   delta: float = 0.0,
                                   h0: str = '==', alpha: float = 0.05, beta: float = 0.20) -> dict:
    """
    独立样本均值z检验
    H0: mean1 - mean2 - delta 与 0 的关系（==, <=, >=）
    :param n1: 左侧样本量
    :param k1: 左侧样本均值
    :param n2: 右侧样本量
    :param k2: 右侧样本均值
    :param delta: 原假设中两均值的差值δ
    :param h0: 原假设中的关系：μ1 - μ2 == δ, μ1 - μ2 <= δ, μ1 - μ2 >= δ
    :param alpha: α，第Ⅰ类错误概率（= 1 - 置信水平）
    :param beta: β，第Ⅱ类错误概率（= 1 - 功效）
    :return: Z检验结果（dict）
    """
    table = [[k1, n1-k1], [k2, n2-k2]]
    p, p1, p2 = (k1 + k2) / (n1 + n2), k1 / n1, k2 / n2
    # 均值差
    diff_mean = k1 / n1 - k2 / n2
    # P值（z落在拒绝域的概率）
    if h0 in ('=', '=='):
        statistic, p_value = fisher_exact(table, alternative='two-sided')
    elif h0 in ('<', '<='):
        statistic, p_value = fisher_exact(table, alternative='greater')
    elif h0 in ('>', '>='):
        statistic, p_value = fisher_exact(table, alternative='less')
    else:
        statistic, p_value = None, None

    return {
        'method': 'fisher_exact_test',
        'method_name': '[独立样本][Fisher精确检验]',
        'field_type': '比率型',
        'statistic': statistic,
        'p_value': p_value,
        'significance': significance_type(p_value),

        'diff': diff_mean,  # 均值的增量 = mean1 - mean2
        'incr_mean': diff_mean,  # 均值的增量 = mean1 - mean2
        'incr_group1': diff_mean * n1,           # 组1的增量 = (mean1 - mean2) * n1
        'incr_group2': diff_mean * n2,           # 组2的增量 = (mean1 - mean2) * n2，如果组2也做干预的话
        'mde': '后续版本补充',  # 均值增量MDE
        'ci': ('后续版本补充', '后续版本补充'),  # 均值增量的置信区间（双尾α）
        'ci_radius': '后续版本补充',  # 均值增量的置信区间半径（双尾α）
        'effect_size': '后续版本补充',  # 效应量（Cohen's d）

        'std': '',  # 池化标准差（两标准差的加权平均值）
        'sem': '',  # 均值的标准误
        'sem0': '',  # 均值的标准误（假设比率相同）
        'sem1': '',  # 均值的标准误（假设比率不同）

        'n1': n1,
        'n2': n2,
        'k1': k1,
        'k2': k2,
        'mean1': k1 / n1,
        'mean2': k2 / n2,

        'test': f'({k1/n1:.4f}) - ({k2/n2:.4f}) {h0} {delta:.4f}',
        'h0': h0,
        'alpha': alpha,
        'beta': beta
    }



from .ab import AB
from .sample_mean import SampleMean
from .sample_prop import SampleProp
from .io.plot import plot_two_mean, plot_diff_mean

__all__ = ['AB', 'SampleMean', 'SampleProp', 'plot_two_mean', 'plot_diff_mean']


from .sample_mean import SampleMean
from .sample_prop import SampleProp

class AB:
    """
    AB-testing
    """
    def __init__(self, data,  days=1, alpha=0.05, beta=0.20, name='AB-testing'):
        self.name = name
        self.days = days
        self.alpha = alpha
        self.beta = beta
        self.data_input = data
        self.group_list = []
        self.data_output = []
        self.report = ''


__all__ = ['AB']



"""
t检验：两独立样本均值之差t检验
"""

from scipy.stats import t as tf
import numpy as np
from abetter.io.format import significance_type


# 两独立样本均值之差的t检验
def mean_t_test_two_ind(n1: int, mean1: float, std1: float,
                        n2: int, mean2: float, std2: float,
                        delta: float = 0.0,
                        h0='==', alpha: float = 0.05, beta: float = 0.20, equal_var: bool = True) -> dict:
    """
    独立样本均值t检验
    :param n1: 左侧样本量
    :param mean1: 左侧样本均值
    :param std1: 左侧样本标准差
    :param n2: 右侧样本量
    :param mean2: 右侧样本均值
    :param std2: 右侧样本标准差
    :param delta: 原假设中两均值的差值δ=0.0 float, optional
    :param h0: 原假设中的关系：string, optional {'==','<=','>='} μ1 - μ2 == δ, μ1 - μ2 <= δ, μ1 - μ2 >= δ
    :param equal_var: 方差是否相等：bool, optional {True, False}
    :param alpha: α，第Ⅰ类错误概率（= 1 - 置信水平） float, optional
    :param beta:  β，第Ⅱ类错误概率（= 1 - 统计功效） float, optional
    :return: t检验结果（dict）
    """
    # Pooled 标准差
    std = np.sqrt(((n1 - 1) * std1 ** 2 + (n2 - 1) * std2 ** 2) / (n1 - 1 + n2 - 1))
    # SEM 样本均值的标准误
    if equal_var:
        # degrees of freedom
        df = n1 + n2 - 2
        # pooled sem
        sem = np.sqrt(1 / n1 + 1 / n2) * std
    else:
        # degrees of freedom
        df = (std1 ** 2 / n1 + std2 ** 2 / n2) ** 2 / (
                (std1 ** 2 / n1) ** 2 / (n1 - 1) + (std2 ** 2 / n2) ** 2 / (n2 - 1)
        )
        # separate sem
        sem = np.sqrt(std1 ** 2 / n1 + std2 ** 2 / n2)
    # 上分位点
    t_upp_2tail_alpha = tf.isf(alpha / 2, df=df)
    t_upp_1tail_alpha = tf.isf(alpha, df=df)
    t_upp_1tail_beta = tf.isf(beta, df=df)
    # 均值差异
    diff_mean = mean1 - mean2
    # t 统计量
    t = (diff_mean - delta) / sem
    # P值（z落在拒绝域的概率）
    if h0 in ('=', '=='):
        p_value = tf.sf(abs(t), df=df) * 2
    elif h0 in ('<', '<='):
        p_value = tf.sf(t, df=df)
    elif h0 in ('>', '>='):
        p_value = tf.cdf(t, df=df)
    else:
        p_value = '错误'
    # 置信区间半径
    ci_diff_mean_radius = t_upp_2tail_alpha * sem
    ci_diff_mean = list(diff_mean + np.array([-1, 1]) * ci_diff_mean_radius)
    # MDE
    mde_mean = (t_upp_2tail_alpha + t_upp_1tail_beta) * sem
    # effect_size
    effect_size = abs(diff_mean) / std
    return {
        'method': 'mean_t_test',
        'method_name': f'[独立样本][t检验][{'方差相等' if equal_var else '方差不等'}]',
        'field_type': '均值型',
        'statistic': t,
        'p_value': p_value,
        'significance': significance_type(p_value),
        'diff': diff_mean,                       # 均值的增量 mean1 - mean2
        'incr_mean': diff_mean,                  # 均值的增量 = mean1 - mean2
        'incr_group1': diff_mean * n1,           # 组1的增量 = (mean1 - mean2) * n1
        'incr_group2': diff_mean * n2,           # 组2的增量 = (mean1 - mean2) * n2，如果组2也做干预的话
        'mde': mde_mean,                         # 均值增量MDE
        'ci': ci_diff_mean,                      # 均值增量的置信区间（双尾α）
        'ci_radius': ci_diff_mean_radius,        # 均值增量的置信区间半径（双尾α）
        'effect_size': effect_size,              # 效应量（Cohen's d）

        'std': std,                              # 池化标准差（两标准差的加权平均值）
        'sem': sem,                              # 均值的标准误
        'df': df,                                # 自由度

        'n1': n1,
        'n2': n2,
        'mean1': mean1,
        'mean2': mean2,
        'std1': std1,
        'std2': std2,

        'test': f'({mean1:.4f}) - ({mean2:.4f}) {h0} {delta:.4f}',
        'h0': h0,
        'alpha': alpha,
        'beta': beta,
        'equal_var': equal_var
    }



"""
Z检验：单样本均值Z检验
"""

from scipy.stats import norm
import numpy as np
from abetter.io.format import significance_type


# 单样本均值的t检验
def mean_z_test_one(n: int, mean: float, std: float,
                    mean0: float, h0: str = '==', alpha: float = 0.05, beta: float = 0.20) -> dict:
    """
    单样本均值的t检验
    :param n:
    :param mean:
    :param std:
    :param mean0:
    :param h0:
    :param alpha:
    :param beta:
    :return:
    """
    # 上分位点
    z_upp_2tail_alpha = norm.isf(alpha / 2)
    z_upp_1tail_alpha = norm.isf(alpha)
    z_upp_1tail_beta = norm.isf(beta)
    # 统计量
    sem = std / np.sqrt(n)
    diff_mean = mean - mean0
    z = diff_mean / sem
    # P值（z落在拒绝域的概率）
    if h0 in ('=', '=='):
        p_value = norm.sf(abs(z)) * 2
    elif h0 in ('<', '<='):
        p_value = norm.sf(z)
    elif h0 in ('>', '>='):
        p_value = norm.cdf(z)
    else:
        p_value = '错误'
    # 置信区间半径
    ci_diff_mean_radius = z_upp_2tail_alpha * sem
    ci_diff_mean = list(mean + np.array([-1, 1]) * ci_diff_mean_radius)
    # MDE
    mde_mean = (z_upp_2tail_alpha + z_upp_1tail_beta) * sem
    # effect_size
    effect_size = abs(mean) / std
    return {
        'method': 'z_test',
        'method_name': '[单样本][Z检验]',
        'field_type': '均值型',
        'statistic': z,
        'p_value': p_value,
        'significance': significance_type(p_value),

        'diff': diff_mean,                       # 均值的增量 = mean - mean0
        'incr_mean': diff_mean,                  # 均值的增量 = mean1 - mean0
        'incr_group1': diff_mean * n,            # 组1的增量 = (mean1 - mean0) * n
        'mde': mde_mean,                         # 均值增量MDE
        'ci': ci_diff_mean,                      # 均值增量的置信区间（双尾α）
        'ci_radius': ci_diff_mean_radius,        # 均值增量的置信区间半径（双尾α）
        'effect_size': effect_size,              # 效应量（Cohen's d）

        'std': std,
        'sem': sem,                              # 均值的标准误

        'n': n,
        'mean': mean,
        'mean0': mean0,

        'test': f'{mean:.4f} {h0} {mean0:.4f}',
        'h0': h0,

        'alpha': alpha,
        'beta': beta
    }



"""
单个样本，样本之差
"""

class Sample:
    """
    单指标样本
    """
    sid = 0

    def __init__(self, n, mean=None, std=None, k=None, alpha=0.05, beta=0.20, h0='==', days=None,
                 field_type=None, field_name=None, group_ratio=None, group_name=None):
        Sample.sid += 1
        self.id = Sample.sid
        self.field_type = field_type
        self.field_name = field_name
        self.group_ratio = group_ratio  # 组分流比例
        self.group_name = group_name
        self.alpha = alpha
        self.beta = beta
        self.days = days
        self.h0 = h0
        self.n = n
        self.mean = mean
        self.std = std
        self.k = k

    def set_alpha(self, alpha):
        self.alpha = alpha

    def set_beta(self, beta):
        self.beta = beta


class SS:
    """
    指标样本的差
    """

    def __init__(self, s1: Sample, s2: Sample):
        self.s1 = s1
        self.s2 = s2
        self.data_all = None
        self.data_best = None
        self.data_flat = None

    def test(self):
        pass

    def set_alpha(self, alpha):
        self.s1.set_alpha(alpha)
        self.s2.set_alpha(alpha)
        self.test()
        return self

    def set_beta(self, beta):
        self.s1.set_beta(beta)
        self.s2.set_beta(beta)
        self.test()
        return self

    def to_clipboard(self):
        self.data_best.to_clipboard()


__all__ = ['Sample', 'SS']



from .two_inde_mean_z_test import mean_z_test_two_ind
from .two_inde_prop_z_test import prop_z_test_two_ind
from .two_inde_mean_t_test import mean_t_test_two_ind
from .two_inde_var_f_test import var_f_test_two_ind
from .two_inde_prop_fisher_exact_test import prop_fisher_exact_test_two_ind


# from abetter import SampleMean, SampleProp


def mean_z_test(s1, s2, alpha: float=None, beta: float=None, h0: str='==', delta: float=0.0) -> dict:
    """
    均值Z检验
    :param s1: 均值型样本
    :param s2: 均值型样本
    :param alpha: α，第Ⅰ类错误概率（= 1 - 置信水平）
    :param beta: β，第Ⅱ类错误概率（= 1 - 功效）
    :param h0: 原假设中的关系：μ1 - μ2 == δ, μ1 - μ2 <= δ, μ1 - μ2 >= δ
    :param delta: 原假设中两均值的差值δ
    :return: 均值Z检验结果（dict）
    """
    _alpha = alpha or s1.alpha
    _beta = beta or s1.beta

    return mean_z_test_two_ind(
        n1=s1.n, mean1=s1.mean, std1=s1.std,
        n2=s2.n, mean2=s2.mean, std2=s2.std,
        alpha=_alpha, beta=_beta,
        h0=h0,
        delta=delta
    )


def prop_z_test(s1, s2, alpha: float=None, beta: float=None, h0: str='==', delta: float=0.0) -> dict:
    """
    比率Z检验
    :param s1: 均值型样本
    :param s2: 均值型样本
    :param alpha: α，第Ⅰ类错误概率（= 1 - 置信水平）
    :param beta: β，第Ⅱ类错误概率（= 1 - 功效）
    :param h0: 原假设中的关系：μ1 - μ2 == δ, μ1 - μ2 <= δ, μ1 - μ2 >= δ
    :param delta: 原假设中两均值的差值δ
    :return: 比率Z检验结果（dict）
    """
    _alpha = alpha or s1.alpha
    _beta = beta or s1.beta

    return prop_z_test_two_ind(
        n1=s1.n, k1=s1.k,
        n2=s2.n, k2=s2.k,
        alpha=_alpha, beta=_beta,
        h0=h0,
        delta=delta
    )

def prop_fisher_test(s1, s2, alpha: float=None, beta: float=None, h0: str='==', delta: float=0.0) -> dict:
    """
    比率Z检验
    :param s1: 均值型样本
    :param s2: 均值型样本
    :param alpha: α，第Ⅰ类错误概率（= 1 - 置信水平）
    :param beta: β，第Ⅱ类错误概率（= 1 - 功效）
    :param h0: 原假设中的关系：μ1 - μ2 == δ, μ1 - μ2 <= δ, μ1 - μ2 >= δ
    :param delta: 原假设中两均值的差值δ
    :return: 比率Z检验结果（dict）
    """
    _alpha = alpha or s1.alpha
    _beta = beta or s1.beta

    return prop_fisher_exact_test_two_ind(
        n1=s1.n, k1=s1.k,
        n2=s2.n, k2=s2.k,
        alpha=_alpha, beta=_beta,
        h0=h0,
        delta=delta
    )

def mean_t_test(s1, s2, alpha: float=None, beta: float=None, h0: str='==', delta: float=0.0, equal_var: bool=True) -> dict:
    """
    均值t检验
    :param s1: 均值型样本
    :param s2: 均值型样本
    :param alpha: α，第Ⅰ类错误概率（= 1 - 置信水平）
    :param beta: β，第Ⅱ类错误概率（= 1 - 功效）
    :param h0: 原假设中的关系：μ1 - μ2 == δ, μ1 - μ2 <= δ, μ1 - μ2 >= δ
    :param delta: 原假设中两均值的差值δ
    :param equal_var: 方差是否相等（默认True）
    :return: 均值t检验结果（dict）
    """
    _alpha = alpha or s1.alpha
    _beta = beta or s1.beta

    return mean_t_test_two_ind(
        n1=s1.n, mean1=s1.mean, std1=s1.std,
        n2=s2.n, mean2=s2.mean, std2=s2.std,
        alpha=_alpha, beta=_beta,
        h0=h0,
        delta=delta,
        equal_var=equal_var
    )


def var_f_test(s1, s2, h0: str='==') -> dict:
    """
    方差齐性检验
    :param s1: 均值型样本
    :param s2: 均值型样本
    :param h0: 原假设中的关系：μ1 - μ2 == δ, μ1 - μ2 <= δ, μ1 - μ2 >= δ
    :return: 方差F检验结果（dict）
    """
    return var_f_test_two_ind(
        n1=s1.n, std1=s1.std,
        n2=s2.n, std2=s2.std,
        h0=h0
    )
